/*----------------------------------------------------------------------*/
/*		mmtimert.c		multi-media timer via thread demo				*/
/*																		*/
/*		Jeff Reid		09JUL03	 16:30									*/
/*----------------------------------------------------------------------*/
#include <windows.h>
#include <stdio.h>

#pragma comment(lib, "winmm.lib")		/* include winmm.lib */

/*----------------------------------------------------------------------*/
/*		data															*/
/*----------------------------------------------------------------------*/
static HANDLE	heT1;					/* event handle */
static HANDLE	htT1;					/* thread handle */

static volatile DWORD	fxT1;			/* flag to indicate thread exit */
static DWORD	dwThreadT1;				/* thread id */

static TIMECAPS sTimeCaps;				/* min and max timer values */
static MMRESULT mmrTimerId;				/* timer id */
static DWORD	dwTicks;				/* tick counter */

/*----------------------------------------------------------------------*/
/*		code															*/
/*----------------------------------------------------------------------*/
static DWORD	WINAPI TimerThread(LPVOID);

/*----------------------------------------------------------------------*/
/*		main															*/
/*----------------------------------------------------------------------*/
DWORD main(DWORD argc, BYTE **argv)
{

/*	create timer event (auto reset) */
	heT1 = CreateEvent(NULL, FALSE, FALSE, NULL);
	if(!heT1){
		printf("can't create event\n");
		goto exit2;}
	
/*	create timer thread, set to higher priority */

	htT1 = CreateThread(NULL, 0, TimerThread, 0, 0, &dwThreadT1);
	if(!htT1){
		printf("can't create thread\n");
		goto exit2;}
	SetThreadPriority(htT1, THREAD_PRIORITY_ABOVE_NORMAL);

/*										** get min and max period */
	timeGetDevCaps(&sTimeCaps, sizeof(sTimeCaps));
	if(sTimeCaps.wPeriodMin != 1){
		printf("1 ms resolution not allowed\n");
		goto exit1;}

	timeBeginPeriod(1);					/* set period to 1ms */

/*										** create 1ms timer to set event */
	mmrTimerId = timeSetEvent(1, 1, heT1, 0, TIME_PERIODIC|TIME_CALLBACK_EVENT_SET);
	if(mmrTimerId == (MMRESULT)0){
        printf("Failed to generate multimedia timer.\n");
		goto exit0;}

    Sleep(1000);						/* sleep and display ticks */
	printf("# ticks = %ld\n", dwTicks);
    Sleep(1000);
	printf("# ticks = %ld\n", dwTicks);
    Sleep(1000);
	printf("# ticks = %ld\n", dwTicks);
    Sleep(1000);
	printf("# ticks = %ld\n", dwTicks);
    Sleep(1000);
	printf("# ticks = %ld\n", dwTicks);

	timeKillEvent(mmrTimerId);			/* stop the timer */

exit0:
    timeEndPeriod(1);					/* restore resolution to default */

exit1:
	if(htT1){							/* close thread */
		fxT1 = 1;
		SetThreadPriority(htT1, THREAD_PRIORITY_ABOVE_NORMAL);
		SetEvent(heT1);
		WaitForSingleObject(htT1, INFINITE);
		CloseHandle(htT1);}
	if(heT1){							/* close event */
		CloseHandle(heT1);}
exit2:
	return(0);
}

/*----------------------------------------------------------------------*/
/*		TimerThread														*/
/*----------------------------------------------------------------------*/
static DWORD WINAPI TimerThread(LPVOID lpvoid)
{
	while(1){
		WaitForSingleObject(heT1, INFINITE);	/* wait for event */
		if(fxT1 != 0)							/* exit if exit flag set */
			return(0);
		dwTicks += 1;}							/* increment ticks */
}

